@@ -1,46 +1,54 @@
 
 =head1 Name
- 
+
 QBit::Application::Model::DB::Table
- 
+
 =head1 Description
- 
+
 Base class for DB tables.
 
 =cut
 
 package QBit::Application::Model::DB::Table;
-$QBit::Application::Model::DB::Table::VERSION = '0.024';
+$QBit::Application::Model::DB::Table::VERSION = '0.025';
 use qbit;
 
 use base qw(QBit::Application::Model::DB::Class);
 
 =head1 RO accessors
- 
+
 =over
- 
+
 =item *
- 
+
 B<name>
 
 =item *
- 
+
 B<inherits>
 
 =item *
- 
+
 B<primary_key>
 
 =item *
- 
+
 B<indexes>
 
 =item *
- 
+
 B<foreign_keys>
 
+=item *
+
+B<collate>
+
+=item *
+
+B<engine>
+
 =back
- 
+
 =cut
 
 __PACKAGE__->mk_ro_accessors(
@@ -73,10 +81,6 @@ B<add>
 
 =item
 
-B<edit>
-
-=item
-
 B<delete>
 
 =item
@@ -96,7 +100,6 @@ __PACKAGE__->abstract_methods(
       create_sql
       add_multi
       add
-      edit
       delete
       _get_field_object
       _convert_fk_auto_type
@@ -110,7 +113,7 @@ __PACKAGE__->abstract_methods(
 B<No arguments.>
 
 Method called from L</new> before return object.
- 
+
 =cut
 
 sub init {
@@ -156,7 +159,7 @@ B<$fields> - reference to array of objects (QBit::Application::Model::DB::Field)
 B<Example:>
 
   my $fields = $app->db->users->fields();
- 
+
 =cut
 
 sub fields {
@@ -182,7 +185,7 @@ B<@field_names>
 B<Example:>
 
   my @field_names = $app->db->users->field_names();
- 
+
 =cut
 
 sub field_names {
@@ -293,6 +296,43 @@ sub get_all {
     return $query->get_all();
 }
 
+=head2 edit
+
+B<Arguments:>
+
+=over
+
+=item *
+
+B<$pkeys_or_filter> - perl variables or object (QBit::Application::Model::DB::filter)
+
+=item *
+
+B<$data> - reference to hash
+
+=back
+
+B<Example:>
+
+  $app->db->users->edit(1, {login => 'LoginNew'});
+  $app->db->users->edit([1], {login => 'LoginNew'});
+  $app->db->users->edit({id => 1}, {login => 'LoginNew'});
+  $app->db->users->edit($app->db->filter({login => 'Login'}), {login => 'LoginNew'});
+
+=cut
+
+sub edit {
+    my ($self, $pkeys_or_filter, $data, %opts) = @_;
+
+    my $query = $self->db->query(comment => $opts{'comment'})->update(
+        table  => $self,
+        data   => $data,
+        filter => $self->_pkeys_or_filter_to_filter($pkeys_or_filter),
+    );
+
+    return $query->do();
+}
+
 =head2 get
 
 B<Arguments:>
@@ -373,7 +413,7 @@ Truncate table.
 B<Example:>
 
   $app->db->users->truncate();
- 
+
 =cut
 
 sub truncate {
@@ -421,7 +461,7 @@ sub drop {
 =head2 default_fields
 
 You can redefine this method in your Model.
- 
+
 =cut
 
 sub default_fields { }
@@ -429,7 +469,7 @@ sub default_fields { }
 =head2 default_primary_key
 
 You can redefine this method in your Model.
- 
+
 =cut
 
 sub default_primary_key { }
@@ -437,7 +477,7 @@ sub default_primary_key { }
 =head2 default_indexes
 
 You can redefine this method in your Model.
- 
+
 =cut
 
 sub default_indexes { }
@@ -445,7 +485,7 @@ sub default_indexes { }
 =head2 default_foreign_keys
 
 You can redefine this method in your Model.
- 
+
 =cut
 
 sub default_foreign_keys { }
